/***********************************************************************
 * Copyright(C) 2006 Valtech Co.,Ltd.
 * All Rights Reserved. This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.opensource.org/licenses/cpl.php
 ***********************************************************************/
package jp.valtech.bts.ui.navigator;

import jp.valtech.bts.ui.BtsPlugin;

import org.eclipse.swt.graphics.Image;



/**
 * 下書き保存の課題票を格納するフォルダノードです。
 * 
 * @author		<A href="mailto:m_sugitou@valtech.jp">M.Sugito</A>
 * @version	Ver.0.8
 */
public class DraftNode extends BtsTreeNode implements IChildCountNode, ISecondNode {
    
	/** このノードのイメージファイル */
	private static final Image IMAGE = BtsPlugin.getInstance().getImage("draft.gif"); 
	
	/** このノードの表示名 */
	private static final String TEXT_NAME = Messages.getString("DraftNode.0");  //$NON-NLS-1$
	
	/** このノードのソートキー */
	private static final String SORT_KEY = "002";

	/** このノードが持つ子ノードの数 */
	private int childCount = 0;

	/**
     * 親ノード・ラベルを設定します。
     * 
     * @param		parent				このノードの親
     */
    public DraftNode(BtsTreeNode parent) {
    	super(parent, TEXT_NAME, IMAGE);
    }
    
	/**
	 * ソートキーを返します。
	 * 
	 * @return		このノードのソートキー
	 */
	public String getFolderSortKey() {
		return SORT_KEY;
	}

	/*
	 *  (非 Javadoc)
	 * @see jp.valtech.bts.ui.navigator.IChildCountNode#getChildCount()
	 */
	public int getChildCount() {
		return childCount;
	}
	
	/**
	 * 子ノードの数をインクリメントします。
	 *
	 */
	public void incrementChildCount() {
		childCount++;
	}
	
	/**
	 * 子ノードの数をデクリメントします。
	 *
	 */
	public void decrementChildCount() {
		childCount--;
	}

	/*
	 *  (非 Javadoc)
	 * @see jp.valtech.bts.ui.navigator.AbstractTreeNode#compareTo(java.lang.Object)
	 */
	public int compareTo(Object obj) {
		if(obj instanceof DraftNode) {
			ISecondNode that = (ISecondNode)obj;
			String thisKey = this.getFolderSortKey();
			String thatKey = that.getFolderSortKey();
			return thisKey.compareTo(thatKey);
		}
		return -1;
	}

}
